using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Find_Laser
{
    public partial class Form1 : Form
    {
        private TIS.Imaging.OverlayBitmap overlayBitmap;
        private TIS.Imaging.FrameSnapSink _fhs;
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Open a device and make some settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load_1(object sender, EventArgs e)
        {
            icImagingControl1.ShowDeviceSettingsDialog();

            if (!icImagingControl1.DeviceValid)
            {
                cmdStartLive.Enabled = false;
                cmdSettings.Enabled = false;
            }

            icImagingControl1.LiveDisplayDefault = false;
            icImagingControl1.LiveDisplaySize = icImagingControl1.Size;

            _fhs = new TIS.Imaging.FrameSnapSink(TIS.Imaging.MediaSubtypes.RGB24);
            icImagingControl1.Sink = _fhs;

            // The overlaybitmap position is set in the display path.
            icImagingControl1.OverlayBitmapPosition = TIS.Imaging.PathPositions.Display;
            overlayBitmap = icImagingControl1.OverlayBitmapAtPath[TIS.Imaging.PathPositions.Display];

            // Enable the overlay.
            overlayBitmap.Enable = true;
            
            // Save current frame to the ImageActiveBuffer.
            //icImagingControl1.LiveCaptureContinuous = true;

            cmdFindLaser.Enabled = false;
            cmdStoplive.Enabled = false;
        }

        /// <summary>
        /// Look for the laser point and draw a crosshair.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdFindLaser_Click(object sender, EventArgs e)
        {
            int xLaser, yLaser;

            // Clear the overlay.
            overlayBitmap.Fill(overlayBitmap.DropOutColor);
            
            if (findLaserPoint(out xLaser, out yLaser))
            {
                DrawCrosshairs(xLaser, yLaser);
            }
            else
            {
                MessageBox.Show("No laser point found!");
            }
        }
        /// <summary>
        /// This method searches the laser point on a target and returns the coordinates.
        /// </summary>
        /// <param name="xLaser">The x-coordinate of the point.</param>
        /// <param name="yLaser">The y-coordinate of the point.</param>
        private bool findLaserPoint(out int xLaser, out int yLaser)
        {
            TIS.Imaging.IFrameQueueBuffer rval = _fhs.SnapSingle(TimeSpan.FromSeconds(1));
            BufferAccessHelper buf = new BufferAccessHelper(rval);

            int redPixel = 0;
            xLaser = 0;
            yLaser = 0;

            
            for (int y = 0; y < rval.FrameType.Height; y++)
            {
                for (int x = 0; x < rval.FrameType.Width; x++)
                {
                    // x and y index for the imagebuffer.
                    
                    int indexXred = x * 3 + 2;
                    int indexXgreen = x * 3 + 1;
                    int indexXblue = x * 3 + 0;
                    int indexY = rval.FrameType.Height - y - 1;
                    
                    if (buf[indexXred, indexY] > 220 & buf[indexXgreen, indexY] < 80 & buf[indexXblue, indexY] < 80)
                    {
                        // Count red pixels.
                        redPixel++;
                        Console.Write(buf[indexXred, indexY]+":" + buf[indexXgreen, indexY] + ":" + buf[indexXblue, indexY] );
                        xLaser = xLaser + x;
                        yLaser = yLaser + y;
                    }
                }
            }
            // Calculate the center of all red pixels.
            if (redPixel > 0)
            {
                xLaser = xLaser / redPixel;
                yLaser = yLaser / redPixel;

                return true;
            }
            else
            {
                return false;
            }
        }

        class BufferAccessHelper
        {
            TIS.Imaging.IFrameQueueBuffer buf;

            public BufferAccessHelper(TIS.Imaging.IFrameQueueBuffer buf)
            {
                this.buf = buf;
            }

            public unsafe byte this[int x, int y]
            {
                get
                {
                    byte* ptr = buf.Ptr + y * buf.FrameType.BytesPerLine + x;
                    return *ptr;
                }
                set
                {
                    byte* ptr = buf.Ptr + y * buf.FrameType.BytesPerLine + x;
                    *ptr = value;
                }
            }
        }
        /// <summary>
        /// This method draws crosshairs on the overplay.
        /// </summary>
        /// <param name="x">x-coordinate of the point where the crosshair is shown.</param>
        /// <param name="y">y-coordinate of the point where the crosshair is shown</param>
        private void DrawCrosshairs(int x, int y)
        {
            overlayBitmap.DrawLine(Color.Red, x, y - 10, x, y + 10);
            overlayBitmap.DrawLine(Color.Red, x - 10, y, x + 10, y);
            overlayBitmap.DrawText(Color.White, x + 3, y + 2, x.ToString() + "," + y.ToString());
        }
        /// <summary>
        /// Start the live video.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdStartLive_Click(object sender, EventArgs e)
        {
            icImagingControl1.LiveStart();
            cmdStartLive.Enabled = false;
            cmdStoplive.Enabled = true;
            cmdFindLaser.Enabled = true;
        }

        /// <summary>
        /// Stop the live video.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdStoplive_Click(object sender, EventArgs e)
        {
            cmdStartLive.Enabled = true;
            cmdStoplive.Enabled = false;
            cmdFindLaser.Enabled = false;

            icImagingControl1.LiveStop();
        }

        /// <summary>
        /// Show Device Settings Dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdSettings_Click(object sender, EventArgs e)
        {
            icImagingControl1.ShowPropertyDialog();
        }

        /// <summary>
        /// Show device settings dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdDevice_Click(object sender, EventArgs e)
        {
            if (icImagingControl1.LiveVideoRunning)
            {
                icImagingControl1.LiveStop();
            }

            icImagingControl1.ShowDeviceSettingsDialog();

            if (icImagingControl1.DeviceValid)
            {
                cmdStartLive.Enabled = true;
                cmdSettings.Enabled = true;
            }
        }

    }
}
